@@ -1,5 +1,6 @@
-from pydantic import BaseModel, Field
+from pydantic import ConfigDict, BaseModel, Field
 from datetime import datetime
+from typing import Optional
 
 
 class ChgcarDataDoc(BaseModel):
@@ -7,20 +8,18 @@ class ChgcarDataDoc(BaseModel):
     Electron charge density metadata for selected materials.
     """
 
-    fs_id: str = Field(
+    fs_id: Optional[str] = Field(
         None, description="Unique object ID for the charge density data."
     )
 
-    last_updated: datetime = Field(
+    last_updated: Optional[datetime] = Field(
         None,
         description="Timestamp for the most recent update to the charge density data.",
     )
 
-    task_id: str = Field(
+    task_id: Optional[str] = Field(
         None,
         description="The Materials Project ID of the calculation producing the charge density data. "
         "This comes in the form: mp-******.",
     )
-
-    class Config:
-        extra = "allow"
+    model_config = ConfigDict(extra="allow")
