@@ -1,4 +1,4 @@
-from typing import Dict, List
+from typing import Dict, List, Optional
 
 from pydantic import BaseModel, Field
 
@@ -8,22 +8,22 @@ class EOSDoc(BaseModel):
     Fitted equations of state and energies and volumes used for fits.
     """
 
-    energies: List[float] = Field(
+    energies: Optional[List[float]] = Field(
         None,
         description="Common energies in eV/atom that the equations of state are plotted with.",
     )
 
-    volumes: List[float] = Field(
+    volumes: Optional[List[float]] = Field(
         None,
         description="Common volumes in A³/atom that the equations of state are plotted with.",
     )
 
-    eos: Dict = Field(
+    eos: Optional[Dict] = Field(
         None,
         description="Data for each type of equation of state.",
     )
 
-    task_id: str = Field(
+    task_id: Optional[str] = Field(
         None,
         description="The Materials Project ID of the material. This comes in the form: mp-******.",
     )
