@@ -1,5 +1,10 @@
 import re
-from typing import Union
+from typing import Union, Any, Callable
+
+from pydantic_core import CoreSchema, core_schema
+
+from pydantic import GetJsonSchemaHandler
+from pydantic.json_schema import JsonSchemaValue
 
 
 mpid_regex = re.compile(r"^([A-Za-z]*-)?(\d+)(-[A-Za-z0-9]+)*$")
@@ -80,24 +85,29 @@ class MPID(str):
         return hash(self.string)
 
     @classmethod
-    def __get_validators__(cls):
-        yield cls.validate
+    def __get_pydantic_core_schema__(
+        cls, source: type[Any], handler: Callable[[Any], core_schema.CoreSchema]
+    ) -> core_schema.CoreSchema:
+        return core_schema.with_info_plain_validator_function(cls.validate)
 
     @classmethod
-    def __modify_schema__(cls, field_schema):
-        field_schema.update(
+    def __get_pydantic_json_schema__(
+        cls, core_schema: CoreSchema, handler: GetJsonSchemaHandler
+    ) -> JsonSchemaValue:
+        return dict(
             pattern=r"^([A-Za-z]*-)?(\d+)(-[A-Za-z0-9]+)*$",
             examples=["mp-3534", "3453", "mp-834-Ag"],
+            type="string",
         )
 
     @classmethod
-    def validate(cls, v):
-        if isinstance(v, MPID):
-            return v
-        elif isinstance(v, str) and mpid_regex.fullmatch(v):
-            return MPID(v)
-        elif isinstance(v, int):
-            return MPID(v)
+    def validate(cls, __input_value: Any, _: core_schema.ValidationInfo):
+        if isinstance(__input_value, MPID):
+            return __input_value
+        elif isinstance(__input_value, str) and mpid_regex.fullmatch(__input_value):
+            return MPID(__input_value)
+        elif isinstance(__input_value, int):
+            return MPID(__input_value)
 
         raise ValueError("Invalid MPID Format")
 
@@ -170,25 +180,30 @@ class MPculeID(str):
         return hash(self.string)
 
     @classmethod
-    def __get_validators__(cls):
-        yield cls.validate
+    def __get_pydantic_core_schema__(
+        cls, source: type[Any], handler: Callable[[Any], core_schema.CoreSchema]
+    ) -> core_schema.CoreSchema:
+        return core_schema.with_info_plain_validator_function(cls.validate)
 
     @classmethod
-    def __modify_schema__(cls, field_schema):
-        field_schema.update(
+    def __get_pydantic_json_schema__(
+        cls, core_schema: CoreSchema, handler: GetJsonSchemaHandler
+    ) -> JsonSchemaValue:
+        return dict(
             pattern=r"^^([A-Za-z]+-)?([A-Fa-f0-9]+)-([A-Za-z0-9]+)-(m?[0-9]+)-([0-9]+)$",
             examples=[
                 "1a525231bdac3f13e2fac0962fe8d053-Mg1-0-1",
-                "22b40b99719ac570fc7e6225e855ec6e-F5Li1P1-m1-2"
+                "22b40b99719ac570fc7e6225e855ec6e-F5Li1P1-m1-2",
                 "mpcule-b9ba54febc77d2a9177accf4605767db-C1H41-2",
             ],
+            type="string",
         )
 
     @classmethod
-    def validate(cls, v):
-        if isinstance(v, MPculeID):
-            return v
-        elif isinstance(v, str) and mpculeid_regex.fullmatch(v):
-            return MPculeID(v)
+    def validate(cls, __input_value: Any, _: core_schema.ValidationInfo):
+        if isinstance(__input_value, MPculeID):
+            return __input_value
+        elif isinstance(__input_value, str) and mpculeid_regex.fullmatch(__input_value):
+            return MPculeID(__input_value)
 
         raise ValueError("Invalid MPculeID Format")
