@@ -2,6 +2,7 @@
 from collections import defaultdict
 from typing import Dict, List, Optional, Union
 from datetime import datetime
+from emmet.core.base import EmmetMeta
 from emmet.core.utils import ValueEnum
 
 from pydantic import BaseModel, Field
@@ -19,15 +20,15 @@ class DecompositionProduct(BaseModel):
     Entry metadata for a decomposition process
     """
 
-    material_id: MPID = Field(
+    material_id: Optional[MPID] = Field(
         None,
         description="The Materials Project ID for the material this decomposition points to.",
     )
-    formula: str = Field(
+    formula: Optional[str] = Field(
         None,
         description="The formula of the decomposed material this material decomposes to.",
     )
-    amount: float = Field(
+    amount: Optional[float] = Field(
         None,
         description="The amount of the decomposed material by formula units this this material decomposes to.",
     )
@@ -45,7 +46,7 @@ class ThermoDoc(PropertyDoc):
     A thermo entry document
     """
 
-    property_name = "thermo"
+    property_name: str = "thermo"
 
     thermo_type: Union[ThermoType, RunType] = Field(
         ...,
@@ -66,9 +67,9 @@ class ThermoDoc(PropertyDoc):
         description="The total corrected DFT energy of this material per atom in eV/atom.",
     )
 
-    energy_uncertainy_per_atom: float = Field(None, description="")
+    energy_uncertainy_per_atom: Optional[float] = Field(None, description="")
 
-    formation_energy_per_atom: float = Field(
+    formation_energy_per_atom: Optional[float] = Field(
         None, description="The formation energy per atom in eV/atom."
     )
 
@@ -81,23 +82,23 @@ class ThermoDoc(PropertyDoc):
         description="Flag for whether this material is on the hull and therefore stable.",
     )
 
-    equilibrium_reaction_energy_per_atom: float = Field(
+    equilibrium_reaction_energy_per_atom: Optional[float] = Field(
         None,
         description="The reaction energy of a stable entry from the neighboring equilibrium stable materials in eV."
         " Also known as the inverse distance to hull.",
     )
 
-    decomposes_to: List[DecompositionProduct] = Field(
+    decomposes_to: Optional[List[DecompositionProduct]] = Field(
         None,
         description="List of decomposition data for this material. Only valid for metastable or unstable material.",
     )
 
-    decomposition_enthalpy: float = Field(
+    decomposition_enthalpy: Optional[float] = Field(
         None,
         description="Decomposition enthalpy as defined by `get_decomp_and_phase_separation_energy` in pymatgen.",
     )
 
-    decomposition_enthalpy_decomposes_to: List[DecompositionProduct] = Field(
+    decomposition_enthalpy_decomposes_to: Optional[List[DecompositionProduct]] = Field(
         None,
         description="List of decomposition data associated with the decomposition_enthalpy quantity.",
     )
@@ -178,6 +179,8 @@ class ThermoDoc(PropertyDoc):
 
             (decomp, ehull) = pd.get_decomp_and_e_above_hull(blessed_entry)
 
+            builder_meta = EmmetMeta(license=blessed_entry.data.get("license"))
+
             d = {
                 "thermo_id": "{}_{}".format(material_id, str(thermo_type)),
                 "material_id": material_id,
@@ -189,6 +192,7 @@ class ThermoDoc(PropertyDoc):
                 "formation_energy_per_atom": pd.get_form_energy_per_atom(blessed_entry),
                 "energy_above_hull": ehull,
                 "is_stable": blessed_entry in pd.stable_entries,
+                "builder_meta": builder_meta.model_dump(),
             }
 
             # Uncomment to make last_updated line up with materials.
@@ -235,7 +239,7 @@ class ThermoDoc(PropertyDoc):
             d["entry_types"] = []
             d["entries"] = {}
 
-            # Currently, each entry group contains a single entry due to how the compatability scheme works
+            # Currently, each entry group contains a single entry due to how the compatibility scheme works
             for entry in entry_group:
                 d["entry_types"].append(entry.parameters.get("run_type", "Unknown"))
                 d["entries"][entry.parameters.get("run_type", "Unknown")] = entry
@@ -293,7 +297,7 @@ class PhaseDiagramDoc(BaseModel):
     A phase diagram document
     """
 
-    property_name = "phase_diagram"
+    property_name: str = "phase_diagram"
 
     phase_diagram_id: str = Field(
         ...,
